@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2004, 2005 Jean-Yves Lefort <jylefort@brutele.be>
+ * Copyright (C) 2004-2006 Jean-Yves Lefort <jylefort@brutele.be>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,8 +25,10 @@
 /*** constants ***************************************************************/
 
 static const GtkStockItem items[] = {
+  /* translators: header capitalization */
   { MN_STOCK_SELECT_ALL, N_("Select _All"), 0, 0, NULL },
-  { MN_STOCK_LEAVE_FULLSCREEN, N_("Leave Fullscreen"), 0, 0, NULL },
+  /* translators: header capitalization */
+  /* translators: header capitalization */
   { MN_STOCK_CONNECT, N_("Co_nnect"), 0, 0, NULL }
 };
 
@@ -43,25 +45,24 @@ mn_stock_init (void)
     const char	*source_stock_id;
   } icons[] = {
     { MN_STOCK_MAIL,			NULL, "stock_mail" },
+    { MN_STOCK_NO_MAIL,			NULL, "stock_inbox" },
     { MN_STOCK_LOCAL,			NULL, "stock_folder" },
     { MN_STOCK_REMOTE,			NULL, "stock_internet" },
     { MN_STOCK_POLLED,			NULL, "stock_timer" },
     { MN_STOCK_UNKNOWN,			NULL, "stock_unknown" },
     { MN_STOCK_ERROR,			NULL, NULL, GTK_STOCK_DIALOG_ERROR },
-#ifdef WITH_GMAIL
-    { MN_STOCK_GMAIL,			MN_IMAGE_FILE(UIDIR, "gmail.png") },
+#if WITH_GMAIL
+    { MN_STOCK_GMAIL,			UIDIR G_DIR_SEPARATOR_S "gmail.png" },
 #endif
-#if defined(WITH_MBOX) || defined(WITH_MH) || defined(WITH_MAILDIR) || defined(WITH_SYLPHEED)
+#if WITH_MBOX || WITH_MH || WITH_MAILDIR || WITH_SYLPHEED
     { MN_STOCK_SYSTEM_MAILBOX,		NULL, "gnome-system" },
 #endif
-#ifdef WITH_EVOLUTION
+#if WITH_EVOLUTION
     { MN_STOCK_EVOLUTION_MAILBOX,	NULL, "evolution" },
 #endif
     { MN_STOCK_SELECT_ALL,		NULL, "stock_select-all" },
-    { MN_STOCK_MAIL_SUMMARY,		MN_IMAGE_FILE(GNOMEPIXMAPSDIR, "mail-notification.png") },
+    { MN_STOCK_MAIL_SUMMARY,		GNOMEPIXMAPSDIR G_DIR_SEPARATOR_S "mail-notification.png" },
     { MN_STOCK_MAIL_READER,		NULL, "stock_mail-handling" },
-    { MN_STOCK_MAIN_WINDOW,		MN_IMAGE_FILE(UIDIR, "main-window.png") },
-    { MN_STOCK_LEAVE_FULLSCREEN,	NULL, "stock_leave-fullscreen" },
     { MN_STOCK_CONNECT,			NULL, NULL, GTK_STOCK_CONNECT }
   };
   GtkIconFactory *factory;
@@ -75,11 +76,11 @@ mn_stock_init (void)
   for (i = 0; i < G_N_ELEMENTS(icons); i++)
     {
       GtkIconSet *icon_set;
-      
+
       if (icons[i].filename)
 	{
 	  GdkPixbuf *pixbuf;
-	  
+
 	  pixbuf = mn_pixbuf_new(icons[i].filename);
 	  if (pixbuf)
 	    {
@@ -92,7 +93,7 @@ mn_stock_init (void)
       else if (icons[i].icon_name)
 	{
 	  GtkIconSource *icon_source;
-	  
+
 	  icon_set = gtk_icon_set_new();
 	  icon_source = gtk_icon_source_new();
 	  gtk_icon_source_set_icon_name(icon_source, icons[i].icon_name);
@@ -110,7 +111,7 @@ mn_stock_init (void)
       gtk_icon_factory_add(factory, icons[i].stock_id, icon_set);
       gtk_icon_set_unref(icon_set);
     }
-  
+
   g_object_unref(factory);
 
   gtk_stock_add_static(items, G_N_ELEMENTS(items));
